/* 
 * tclWin32Dll.c --
 *
 *	This file contains the DLL entry point which sets up the 32-to-16-bit
 *	thunking code for SynchSpawn if the library is running under Win32s.
 *
 * Copyright (c) 1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

static char sccsid[] = "@(#) tclWin32Dll.c 1.3 95/09/14 19:52:15";

#define W32SUT_32    // Needed for w32sut.h in 32-bit code

#include <windows.h>
#include <w32sut.h>
#include "tclWinInt.h"

typedef BOOL (WINAPI * PUTREGISTER) ( HANDLE hModule,
	LPCSTR     SixteenBitDLL,
	LPCSTR     InitName,
	LPCSTR     ProcName,
	UT32PROC * ThirtyTwoBitThunk,
	FARPROC    UT32Callback,
	LPVOID     Buff
    );

typedef VOID (WINAPI * PUTUNREGISTER) (HANDLE hModule);

typedef DWORD (APIENTRY *PUT32CBPROC) (LPVOID Buff, DWORD UserDefined);

UT32PROC      tclUTProc = NULL;
PUTREGISTER   tclUTRegister = NULL;
PUTUNREGISTER tclUTUnRegister = NULL;
PUT32CBPROC   tclUT32CBProc = NULL;
int           tclProcessesAttached = 0;
BOOL          tclWin32s = FALSE;
HANDLE        tclKernel32 = 0;


/*
 *----------------------------------------------------------------------
 *
 * DllEntryPoint --
 *
 *	DLL entry point. Establishes thunk.
 *
 * Results:
 *	TRUE on sucess, FALSE on failure.
 *
 * Side effects:
 *	Establishes 32-to-16 bit thunk.
 *
 *----------------------------------------------------------------------
 */
BOOL APIENTRY DllEntryPoint(HINSTANCE hInst, DWORD Reason, LPVOID Reserved)
{
   DWORD Version;
   BOOL result;

   if ( Reason == DLL_PROCESS_ATTACH ) {
       /*
	* Registration of UT need to be done only once for first
	* attaching process.  At that time set the fWin32s flag
	* to indicate if the DLL is executing under Win32s or not.
	*/

       if( tclProcessesAttached++ ) {
	   return(TRUE);         /* Not the first initialization. */
       }

       /* Find out if we're running on Win32s */
       Version = GetVersion();
       tclWin32s = (BOOL) (!(Version < 0x80000000))
	   && (LOBYTE(LOWORD(Version)) < 4);

       if( !tclWin32s )
	   return(TRUE);    /* Win32s - no further initialization needed */

       tclKernel32 = LoadLibrary( "Kernel32.Dll" ); /* Get Kernel32.Dll
						       handle */

       tclUTRegister = (PUTREGISTER) GetProcAddress( tclKernel32,
	       "UTRegister");


       if( !tclUTRegister ) {
	   return(FALSE);        /* Error- Win32s, but can't find UTRegister */
       }

       tclUTUnRegister = (PUTUNREGISTER) GetProcAddress(tclKernel32,
	       "UTUnRegister");
       
       if( !tclUTUnRegister ) {
	   return(FALSE);    /* Error- Win32s, but can't find
				tclUTUnRegister */
       }


       result = (*tclUTRegister)( hInst,        /* TCL32.DLL module handle */
	       "TCL16.DLL",                     /* 16-bit thunk dll */
	       "UTInit",                        /* init routine */
	       "UTProc",                        /* 16-bit dispatch routine */
	       &tclUTProc,                      /* Receives thunk address */
	       (FARPROC)tclUT32CBProc,          /* callback function */
	       NULL );                          /* no shared memory */
   }
   if((Reason == DLL_PROCESS_DETACH) && (0 == --tclProcessesAttached) &&
	   tclWin32s) {
       (*tclUTUnRegister)( hInst );
       FreeLibrary( tclKernel32 );
   }
   return TRUE; 
}


/*
 *----------------------------------------------------------------------
 *
 * TclSynchSpawn --
 *
 *	32-bit entry point to the 16-bit SynchSpawn code.
 *
 * Results:
 *	1 on sucess, 0 on failure.
 *
 * Side effects:
 *	Spawns a command and waits for it to complete.
 *
 *----------------------------------------------------------------------
 */
DWORD APIENTRY
TclSynchSpawn( LPCSTR CmdLine, UINT CmdShow )
{
   DWORD Args[2];
   PVOID Translist[2];

   Args[0] = (DWORD) CmdLine;
   Args[1] = (DWORD) CmdShow;

   Translist[0] = &Args[0];
   Translist[1] = NULL;

   return( (*tclUTProc)( Args, TCLSYNCHSPAWN, Translist) );
}
